Type: Package
Package: SpaceMarkers
Title: Spatial Interaction Markers
Version: 2.1.0
Authors@R: c(
    person(given = "Atul", family = "Deshpande", email = "adeshpande@jhu.edu", role = c("aut", "cre"), comment = c(ORCID="0000-0001-5144-6924")),
    person(given = "Ludmila", family = "Danilova", email = "ldanilo1@jhmi.edu", role = "ctb"),
    person(given = "Dmitrijs", family = "Lvovs", email = "dlvovs1@jh.edu", role = "ctb", comment = c(ORCID = "0009-0003-2152-6853"))
  )
BugReports: https://github.com/DeshpandeLab/SpaceMarkers/issues
URL: https://github.com/DeshpandeLab/SpaceMarkers
Description: Spatial transcriptomic technologies have helped to resolve the connection
            between gene expression and the 2D orientation of tissues relative to each other.
            However, the limited single-cell resolution makes it difficult to highlight the most 
            important molecular interactions in these tissues. SpaceMarkers, R/Bioconductor software,
            can help to find molecular interactions, by identifying genes associated with latent space 
            interactions in spatial transcriptomics.
Depends: R (>= 4.4.0)
biocViews: SingleCell, GeneExpression, Software, Spatial,
        Transcriptomics
Imports: matrixStats, matrixTests, rstatix, spatstat.explore,
        spatstat.geom, ape, hdf5r, nanoparquet, jsonlite, Matrix,
        qvalue, stats, utils, methods, ggplot2, reshape2, RColorBrewer,
        circlize, mixtools, dplyr, readbitmap, rlang, effsize, viridis
Suggests: data.table, devtools, knitr, cowplot, rjson, rmarkdown,
        BiocStyle, testthat (>= 3.0.0), CoGAPS, ComplexHeatmap
Enhances: BiocParallel
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: false
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
License: MIT + file LICENSE
git_url: https://git.bioconductor.org/packages/SpaceMarkers
git_branch: devel
git_last_commit: 2b359d8
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 23:17:28 UTC; biocbuild
Author: Atul Deshpande [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5144-6924>),
  Ludmila Danilova [ctb],
  Dmitrijs Lvovs [ctb] (ORCID: <https://orcid.org/0009-0003-2152-6853>)
Maintainer: Atul Deshpande <adeshpande@jhu.edu>
