% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{lrdf}
\alias{lrdf}
\title{Curated Ligand-receptor interaction genes
A list of vectors with genes associated with ligand-receptor interactions from CellChat database}
\format{
A data frame with LR interaction genes
}
\value{
A list of vectors of LR genes
}
\description{
Curated Ligand-receptor interaction genes
A list of vectors with genes associated with ligand-receptor interactions from CellChat database
}
