% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessing.R
\name{.read_format}
\alias{.read_format}
\title{readFormat
Reads a format into an R object}
\usage{
.read_format(path)
}
\description{
readFormat
Reads a format into an R object
}
\keyword{internal}
